#include <uart.h>
#include <absacc.h>
#include <reg167.h>
#include <stdio.h>

sbit P2_0 = P2^0;                      		// declare portpin to use              
sbit DP2_0 = DP2^0;                    		// declare direction bit to use        

unsigned char WriteBuffer[] = {"Hello World at UART\r\n"};
unsigned char ReadBuffer[100];
unsigned char trigger = 1;

unsigned char  *readpointer = NULL;
unsigned char  *writepointer = NULL;

void wait (void)  
{                   						// wait function 
  	;                                   	// only to delay for LED flashes 
}

void main (void)  
{

  IEN = 0;									// disable all interrups

  DP2_0 = 1;								// Port 2.0 direction is output

  Ser_1_Init();								// Initialisieren der 1. seriellen Schnittstelle
  
  printf("\n Serielle Schnittstelle 1 initialisiert  \n");

  SetUartBaudrate (1);						// Baudrate des externen UART einstellen
  SetUartInterrupt(ON);						// UART interrupt active
  readpointer = ReadBuffer;

  printf(" UART 16C550c als Schnittstelle 2 initialisiert \n");

  CCM2  = 0x01;								//Capture/Compare auf Capture einstellen (Interrupt-Handling)
  CC8IC = 0x44;								//Interrupt Control-Register einstellen

  CC8IR = 0;								// del. interruptflag 
  IEN = 1;									// enable all interrups
		
    while (1)
  	{
	LEDflasher();
	Write_Ser_2();
    }

}

/********************************************************************************************/
/*	Funktion	:																			*/	
/*																							*/
/*	Ser_1_Init( void )																		*/
/*																							*/
/*	Initialisieren der 1. seriellen Schnittstelle											*/	
/*																							*/
/*																							*/
/*																							*/
/*																							*/
/*																							*/
/********************************************************************************************/

void Ser_1_Init(void)
	{

	P3  |= 0x0400;                       // set port 3.10 output latch (TXD)    
  	DP3 |= 0x0400;                       // configure port 3.10 for output      
                                         // operation. ( TXD output)            
  	DP3 &= 0xF7FF;                       // configure port 3.11 for input       
                                         // operation. ( RXD input)             
  	S0TIC  = 0x80;                       // set transmit interrupt flag         
  	S0RIC  = 0x00;                       // delete receive interrupt flag       
  	S0BG   = 0x80;                       // set baudrate to 9600 baud           
  	S0CON  = 0x8011;                     // set serial mode                     

	putchar(' ');  
	}

/********************************************************************************************/
/*	Funktion	:																			*/	
/*																							*/
/*	Write_Ser_2 ( void )																	*/
/*																							*/
/*	Schreiben von Daten auf die 2. srielle Schnittstelle									*/	
/*																							*/
/*																							*/
/*																							*/
/*																							*/
/*																							*/
/********************************************************************************************/

void Write_Ser_2 (void)
	{
	IEN = 0;
	if (trigger == 2)
	 	{
		writepointer = ReadBuffer;						// Initialisieren der Pointers zum Schreiben
		readpointer = ReadBuffer;
		trigger = 0;
		}
	else
		{
		writepointer = WriteBuffer;
		trigger = 0;
		}

	Transmitter_Holding_Register = *writepointer;		// Zeichen auf UART schreiben
	writepointer++;										// Auslsen eines Interrupts zum Fortfhren 
												        // des Schreibens durch die Interruptroutine
    IEN = 1;
	}



/********************************************************************************************/
/*	Funktion :																				*/
/*																							*/
/*	Set_Baudrate ( unsigned char Baudrate )													*/
/*																							*/
/*  Initialisieren der Baudrate des externen UART mit einer Quarzfrequenz von				*/
/*  3.6864 MHz																				*/
/*																							*/	
/*  Baudrate :  56000	->	3																*/	
/*				38400	->  2																*/
/*				19200	->	1																*/
/*				 9600	->  0																*/
/*																							*/
/********************************************************************************************/

void SetUartBaudrate (unsigned char baudrate)
	{
	static unsigned char DivisorLatchLSB[] = {0x18, 0x0C, 0x06, 0x04};	
	static unsigned char DivisorLatchMSB[] = {0x00, 0x00, 0x00, 0x00};

    Line_Control_Register=Line_Control_Register|0x80;                   // Bit 7 des Line Control Registers setzen
    Divisor_Latch_LSB = DivisorLatchLSB [baudrate];                     // Unteres Baudratenregister
    Divisor_Latch_MSB = DivisorLatchMSB [baudrate];                     // Oberes Baudratenregister
    Line_Control_Register=Line_Control_Register & (~0x80);              // Bit 7 des Line Control Registers zurcksetzen

    Line_Control_Register=0x03;											// UART auf 8-Bit Daten, 1 Stopbit, keine Paritt einstellen
	}

/*****************************************************************************/
/*																			 */
/*	Funktion :																 */
/*																			 */
/*	SetUartReceiveInterrupt ( )												 */
/*																			 */
/*	Initialisieren des Interruptregister des externen Uart					 */
/*																			 */
/*****************************************************************************/

void SetUartInterrupt ()
	{
	Modem_Control_Register=0x00;					// UART ohne Hardware-Handshake
    FIFO_CONTROL_REGISTER=0x00;						// 
	Interrupt_Enable_Register = 0x03;				// Sende- und Empfangsinterrupt aktivieren
	}


/******************************************************************************/
/* Funktionsname     : serial2isr                                             */
/******************************************************************************/
/* Beschreibung      : interrupt gesteuertes Lesen und Schreiben der          */
/*                     seriellen Schnittstelle 2                              */
/******************************************************************************/
/* bergabeparameter                                                          */
/*----------------------------------------------------------------------------*/
/* keine             :                                                        */
/******************************************************************************/
/* Rckgabeparameter                                                          */
/*----------------------------------------------------------------------------*/
/* keine             :                                                        */
/******************************************************************************/

void serial2isr(void) interrupt 0x18
	{ 	                                        
  	unsigned char interruptflag=0;

  	interruptflag = Interrupt_Ident_Register; 

	if ( trigger == 1 )										// Nach senden des letzten Zeichens -> Verlassen	
		{													// der Interruptfunktion ohne Ausgabe
		trigger = 0;
		return;
		}

  	if(interruptflag&0x04)              					// Abfragen, ob Sende- oder Empfangsinterrupt
    	{	
	  	*readpointer = Receiver_Buffer_Register;   			// Zeichen von UART einlesen

		if (*readpointer == '\n')
			 trigger = 2;									// Event fr Eingangspuffer 

	    readpointer++;
   		}

	else if (interruptflag&0x02)
    {
		if (*writepointer == '\n')
  			trigger = 1;

		Transmitter_Holding_Register = *writepointer;		// Zeichen auf UART schreiben
    	writepointer++;
    }
}

/*****************************************************************************/
/*																			 */
/*	Funktion :																 */
/*																			 */
/*	LEDflasher ( )															 */
/*																			 */
/*	Blinken der LED an Port P2.0											 */
/*																			 */
/*****************************************************************************/

 void LEDflasher (void)
	{
	unsigned long i;

    P2_0 = 0;                          // output to LED port                
    for (i=0; i<950000; i++) {         // delay for 150000 counts             
    wait ();                         // call wait function                  
      }

    P2_0 = 1;                          // output to LED port                  
    for (i=0; i<950000; i++) {         // delay for 150000 counts             
      wait ();                         // call wait function                  
      }
	}

